﻿; (function (scope) {

    let operatorConfigs = [
        { type: '=', nb_inputs: 1, multiple: false, apply_to: ['string', 'number', 'datetime', 'boolean'] },
        { type: '!=', nb_inputs: 1, multiple: false, apply_to: ['string', 'number', 'datetime', 'boolean'] },
        { type: 'in', nb_inputs: 1, apply_to: ['string', 'number', 'datetime', 'boolean'] },
        { type: 'notIn', nb_inputs: 1, apply_to: ['string', 'number', 'datetime', 'boolean'] },
        { type: 'contains', nb_inputs: 1, apply_to: ['string'] },
        { type: 'notContain', nb_inputs: 1, apply_to: ['string'] },

        { type: '>', nb_inputs: 1, apply_to: ['string', 'number', 'datetime'] },
        { type: '>=', nb_inputs: 1, apply_to: ['string', 'number', 'datetime'] },
        { type: '<', nb_inputs: 1, apply_to: ['string', 'number', 'datetime'] },
        { type: '<=', nb_inputs: 1, apply_to: ['string', 'number', 'datetime'] },
        { type: 'isNull', nb_inputs: 0, apply_to: ['string', 'number', 'datetime', 'boolean'] },
        { type: 'isNotNull', nb_inputs: 0, apply_to: ['string', 'number', 'datetime', 'boolean'] },
    ];

    

    function buildQueryBuilderFilter(atFilterConfig) {

        return {
            id: atFilterConfig.field,
            label: atFilterConfig.title || atFilterConfig.field,
            operators: atFilterConfig.operators || ['=', '!=', 'in', 'notIn', 'contains', 'notContain', '>', '>=', '<', '<=', 'isNull', 'isNotNull'],
            input: function (rule, name) {
                //let operator = rule.operator.type;
                //var $container = rule.$el.find('.rule-value-container');
                let html = infoGet.buildElementHtml(atFilterConfig.ig_attr);
                return html;
            },
            valueGetter: function (rule) {
                let igElem = rule.$el.find('info-get');
                let je = igElem.children('*[ig_class]');
                let widget = infoGet.getWidget(je[0]);
                return widget.getValue();
            },
            valueSetter: function (rule, value) {
                let igElem = rule.$el.find('info-get');
                let je = igElem.children('*[ig_class]');
                let widget = infoGet.getWidget(je[0]);
                widget.setValue(value);
                //rule.operator.nb_inputs
            },
        };
    }


    function buildQueryBuilderOptions(atFilterConfigs) {
        let filters = atFilterConfigs.map(buildQueryBuilderFilter);

        var options = {
            allow_empty: true,

            default_filter: filters[0]?.id,
            //sort_filters: true,

            plugins: {
                'bt-tooltip-errors': { delay: 100 },
                'sortable': null,
                'filter-description': { mode: 'bootbox' },
                //      'chosen-selectpicker': null,
                'unique-filter': null,
                'bt-checkbox': { color: 'primary' },
                'invert': null,
                //'not-group': null,
            },

            operators: operatorConfigs,
            filters: filters,
        };

        return options;
    }




    /**
     * 
    
    // atFilterConfig:
    {
        "title": "User Name",
        "field": "name",
        "operators": ['=', '!=', 'in', 'notIn', 'contains', 'notContain', '>', '>=', '<', '<=', 'isNull', 'isNotNull'],
        "ig_attr": {
            "ig_class": "TextArea",
            "ig_param": { "width": 300 }
        }
    }
     
     */



    class FilterGrid {

        atFilterConfigs = [];
        $el;
        constructor($el, controller) {

            // #1 get filterConfigs
            let atFields = controller.atFields;
            for (let atField of atFields) {
                let filters = atField.list_getAtFilterConfigs();
                if (!filters?.length) continue;
                for (let filter of filters)
                    this.atFilterConfigs.push(filter);
            }

            if (!this.atFilterConfigs?.length) return;

            let options = buildQueryBuilderOptions(this.atFilterConfigs);

            // init
            this.$el = $el;
            this.$el.queryBuilder(options);
        }


        getRule() {
            if (!this.$el) return null;

            // {field:'',operator:'notContain',value:''}
            // {condition:'not|and|or',rules:[]}
            let rule = this.$el.queryBuilder('getRules');
            rule = this.convertRule(rule);
            return rule;
        }

        // convert {field:'',operator:'notContain',value:''} to {condition:'not',field:'',operator:'contains',value:''}
        convertRule(rule) {
            if (!rule) return null;
            if (rule.operator == 'notContain') {
                return { ...rule, condition: 'not', operator: 'contains' };
            }
            if (rule.rules?.length) {
                rule.rules = rule.rules.map(rule => this.convertRule(rule));
            }
            return rule;
        }

        setRule(rule) {
            if (!this.$el) return;
            this.$el.queryBuilder('setRules', rule);
        }

        clear() {
            if (!this.$el) return;

            this.$el.queryBuilder('setRules', {
                condition: 'AND',
                rules: []
            });
        }

        saveRule() {
            if (!this.$el) return;

            let url = 'FilterCache:'+location.href;
            let rule = this.$el.queryBuilder('getRules');
            localStorage.setValue(url, rule);
        }

        loadRule() {
            if (!this.$el) return;

            let url = 'FilterCache:' + location.href;
            let rule = localStorage.getValue(url);
            if (rule) {
                this.$el.queryBuilder('setRules', rule);
            }
        }


    }


    scope.FilterGrid = FilterGrid;



    // invert
    $.fn.queryBuilder.defaults({
        operatorOpposites: {
            '=': '!=',
            '!=': '=',

            'in': 'notIn',
            'notIn': 'in',

            'contains': 'notContain',
            'notContain': 'contains',

            '>': '<=',
            '<=': '>',

            '<': '>=',
            '>=': '<',

            'isNull': 'isNotNull',
            'isNotNull': 'isNull',
        },

        //conditionOpposites: {
        //    'AND': 'OR',
        //    'OR': 'AND'
        //}
    });



})(window);